import java.util.*;
import java.io.*;

/** This class stores information about a series of Levels, for use by WillPang.
 * It also contains a range of InBuilt scenarios that can be loaded
 * This class is a public release, so that third parties can design WillPang levels
 * until William Denniss impliments a plain text WillPang Scenario file.
 * This document may be freely distributed, so long as it is with a copy of WillPang
 *
 * To add a level, simply mimic what I have done, but with your settings.
 * You will need the java SDK installed on your system which can be downloaded from java.sun.com
 * To impliment your changes, compile this document ("javac Scenario.java")
 * And replace the one that is shipped with WillPang.
 *
 * Please send any levels you make to WillPangLevels@tanksoftware.com
 * And I will add them to any future releases of WillPang (so long as they are OK).
 *
 * Warning:  The Level structure is subject to change.  So your code may only be compatable with the
 * version of WillPang you wrote it in.  The best way is to send your code into me, and I will
 * keep it up to date.
 *
 *@author William Denniss
 *@version 5 June 2001 (WillPang v1.4)
 */

public class Scenario {


	//Instance variables
	private LinkedList levels = new LinkedList();
	private int currentLevelIndex = -1;
	public Level currentLevel;
	public int startingLives;

	
	private int freeLiveAtPoints [];
	public int currentGoal = -1;
	private int freeLivesIndex = 0;
	public String title = "";
	
	public Scenario (int startingLives){
		this.startingLives = startingLives;
	}
	public Scenario (int startingLives, int [] freeLiveAtPoints){
		this.startingLives = startingLives;
		this.freeLiveAtPoints = freeLiveAtPoints;
	}

	// return the number of levels
	public int size () {
		return levels.size();
	}

	public void loadNextFree () {
		try {
			currentGoal = freeLiveAtPoints[freeLivesIndex];
			freeLivesIndex++;
		} catch (ArrayIndexOutOfBoundsException e) {
			//end has been hit
			currentGoal = -1;
		
		}
	}
	
	// return the current level number (-1 indicates that no levels have been loaded)
	public int getCurrentIndex () {
		return currentLevelIndex;
	}


	/** This will add a level to the scenario with only the basic options as detailed below
	 *@param tillNext The number of points before the player advances to the next level
	 *@param ballList A list of Ball objects that will be added to the game when the level loads
	 *@param name The name of the level
	 *@param sizeOfBallDrops The size (1-4) of the ball drops
	 *@param pauseTime the number of seconds before the level will load
	 */
	public void addBasicLevel (long tillNext, Ball [] ballList, String name, int sizeOfBallDrops, long timeBetweenBallDrops, int pauseTime) {
		levels.add(new Level(tillNext, ballList, name, sizeOfBallDrops, timeBetweenBallDrops, pauseTime));
	}

	/** Adds a Level Object to the senario
	 */
	public void addLevel (Level newLevel) {
		levels.add(newLevel);
	}
	
	/** Get the current Level
	 */
	public Level getCurrentLevel () {
		return currentLevel;
	}
	
	/** Is the current scenario at the last level?
	 */
	public boolean atEnd () {
		return currentLevelIndex == levels.size();
	}
	
	/** Load the next level into the game
	 */
	public void loadNextLevel() {
		currentLevelIndex++;
		currentLevel = (Level) levels.get(currentLevelIndex);
	}


	public static Scenario loadLevelFile (String filename, double scale) {
		
		// The scenario that will be loaded
		Scenario loadedScenario;
		
		// Counts the number of lines
		int lineCount = 0;
		
		// Catch any IO errors
		try {
		
			// Open the file
			BufferedReader br = new BufferedReader(new FileReader(filename));
		
			// Load the name of the Scenario
			String scenarioName = br.readLine(); lineCount++;
			
			// Read the lives
			int lives;	
			lives = Integer.parseInt(br.readLine()); lineCount++;
			
			// Read the bonus lives array
			String [] freeLivesString = Delegate.stringCarver(br.readLine(), ","); lineCount++;
			int [] freeLives = new int [freeLivesString.length];
			for (int i = 0; i < freeLives.length; i++) {
				freeLives[i] = Integer.parseInt(freeLivesString[i]);
			}
			
			// Create a new scenario
			loadedScenario = new Scenario(lives, freeLives);

			// Set the name
			loadedScenario.title = scenarioName;

			// Contins the current line as read from th efile
			String line;
			
			// Contains all of the balls since the last level
			LinkedList ballsToDate = new LinkedList();
			
			// Contains the first string of the line (before the first " ");
			String prefix;
			
			// Contains the rest of the line
			String suffix;
			
			// Contains all of the elements of the rest of the line, as separated by ",";
			String [] suffixCarved;
			
			// Get the line
			line = br.readLine(); lineCount++;

			// Loop untill then end of the file
			while (line != null) {
				
				// read the prefix
				prefix = Delegate.carvePos(line, 0, " ");

				// read the suffix
				suffix = Delegate.carvePos(line, 1, " ");

				// carve the suffix
				suffixCarved = Delegate.stringCarver(suffix, ",");
				
				
				// If the prefix is B, then it is a ball
				// the current ball gets added to the ballList LinkedList
				if (prefix.equals("B")) {
					ballsToDate.add(new Ball(
											Integer.parseInt(suffixCarved[0]),
											Integer.parseInt(suffixCarved[1]),
											Integer.parseInt(suffixCarved[2]),
											Integer.parseInt(suffixCarved[3]),
											Integer.parseInt(suffixCarved[4]),
											scale));
				}
				
				// If the prefix is L, then it is a Level
				// The ball LinkedList is read, then reset for the next level
				// and the Level is created using the parsed perameters
				if (prefix.equals("L")) {
				
					// the new ballList array
					Ball [] ballList = new Ball [ballsToDate.size()];
					
					// convert the LinkedList into an array
					if (ballList.length != 0) {
						for (int i = 0; i < ballList.length; i++) {
						
							ballList[i] = (Ball) ballsToDate.get(i);
						
						}
					} else {
						ballList = null;
					}
					
					// Reset the balls for the next level
					ballsToDate = new LinkedList();			
					
					// Add the level to the Scenario
					loadedScenario.addLevel(new Level(
											Long.parseLong(suffixCarved[0]),
											Long.parseLong(suffixCarved[1]),
											ballList,
											suffixCarved[2],

											Long.parseLong(suffixCarved[3]),
											Integer.parseInt(suffixCarved[4]),											
											Integer.parseInt(suffixCarved[5]),
											Integer.parseInt(suffixCarved[6]),	

											Integer.parseInt(suffixCarved[7]),
											Integer.parseInt(suffixCarved[8]),											
											Integer.parseInt(suffixCarved[9]),
											Integer.parseInt(suffixCarved[10]),	


											Long.parseLong(suffixCarved[11]),
											Long.parseLong(suffixCarved[12]),
											suffixCarved[13].equals("true"),
											Integer.parseInt(suffixCarved[14]),	

											suffixCarved[15].equals("true"),
											Integer.parseInt(suffixCarved[16]),	
											Integer.parseInt(suffixCarved[17]),	

											Integer.parseInt(suffixCarved[18]),	
											Integer.parseInt(suffixCarved[19]),	
											Integer.parseInt(suffixCarved[20]),	
											Integer.parseInt(suffixCarved[21]),	
											Integer.parseInt(suffixCarved[22]),	
											Long.parseLong(suffixCarved[23])));

				
				}

				// Read the next line				
				line = br.readLine(); lineCount++;
			}
		
			// Return the Scenario as read
			return loadedScenario;

		} catch (Exception e) {
		
			// If there were problems with the file, return null and output an error message
			if (lineCount != 0) 
				System.out.println("There is an error loading the level " + filename + ". The error occured at line: " + lineCount);
			return null;
		}	
	
		
	}

	// *** START OF INBUILT LEVEL DATA ***
	// The following code has no effect on the running of WillPang except that to load inbuilt levels

	//This is a static class which returns a LevelList object with the objects defined within
	public static Scenario inbuilt(String name, double scale){
		Scenario list = null;// = new Scenario(0);
		Ball[] ballList;
		

		// For reference
		// public Level (long pointsTillNext, long timeTillNext, Ball [] ballList, String name,
		//		long timeBetweenBallDrops, int direction, int minSizeOfBallDrops, int maxSizeOfBallDrops,
		//		int minXBallDrop, int maxXBallDrop, int minYBallDrop, int maxYBallDrop,
		//		long pauseTime, long startPauseTime, boolean whipeBalls, int maxHarpoons,
		//		boolean accumulative, int character1StartPos, int newBallPause,
		//		int currentWeaponType, int harpoonHeadFreezeTime, int backgroundIndex, int startLevelSoundIndex, int endLevelSoundIndex, long timeVarient) {
		
		//public Ball (int x, int y, int ballSize, int initialVelocity, int horDirection, double scale) {
		
		
		if (name.equals("inbuilt1")) {
		
			int [] bonusLives = {500, 2000, 5000, 10000, 13000} ;
			
			list = new Scenario(3, bonusLives);
			
			list.title = "inbuilt1";
			
			ballList = new Ball [1];
			ballList[0] = (new Ball(400,20,4,0,1, scale));
			
			list.addLevel(new Level(4250, -1, ballList, "Welcome to Australia",
			999999, 556556, 0, 0,
			30, 300, 10, 10,
			500, 3000, true, 1,
			false, 0, 1000,
			0, 5000, 556556, 556556, 556556, 10000));
	
			
			list.addLevel(new Level(-1, 50000, null, "Melbourne",
			3000, 556556, 1, 1,
			30, 300, 10, 10,
			500, 3000, true, 5,
			false, 0, 1000,
			2, 5000, 556556, 556556, 556556, 10000));

			ballList = new Ball [4];
			ballList[0] = (new Ball(0,20,4,0,1, scale));
			ballList[1] = (new Ball(100,20,4,0,-1, scale));
			ballList[2] = (new Ball(200,20,4,0,1, scale));
			ballList[3] = (new Ball(300,20,4,0,-1, scale));


			list.addLevel(new Level(4250 * 4, -1, ballList, "Sydney",
			999999, 556556, 0, 0,
			30, 300, 10, 10,
			500, 3000, true, 3,
			false, 0, 10000,
			1, 5000, 556556, 556556, 556556, 10000));

			ballList = new Ball [24];
			ballList[0] = (new Ball(0,20,2,0,1, scale));
			ballList[1] = (new Ball(20,20,2,0,-1, scale));
			ballList[2] = (new Ball(40,20,2,0,1, scale));
			ballList[3] = (new Ball(60,20,2,0,-1, scale));
			ballList[4] = (new Ball(80,20,2,0,1, scale));
			ballList[5] = (new Ball(100,20,2,0,-1, scale));
			ballList[6] = (new Ball(120,20,2,0,1, scale));
			ballList[7] = (new Ball(140,20,2,0,-1, scale));
			ballList[8] = (new Ball(0,200,2,0,1, scale));
			ballList[9] = (new Ball(20,200,2,0,-1, scale));
			ballList[10] = (new Ball(40,200,2,0,1, scale));
			ballList[11] = (new Ball(60,200,2,0,-1, scale));
			ballList[12] = (new Ball(80,200,2,0,1, scale));
			ballList[13] = (new Ball(100,200,2,0,-1, scale));
			ballList[14] = (new Ball(120,200,2,0,1, scale));
			ballList[15] = (new Ball(140,200,2,0,-1, scale));

			ballList[16] = (new Ball(0,40,2,0,-1, scale));
			ballList[17] = (new Ball(0,80,2,0,1, scale));
			ballList[18] = (new Ball(0,120,2,0,-1, scale));
			ballList[19] = (new Ball(0,160,2,0,1, scale));
			ballList[20] = (new Ball(140,40,2,0,-1, scale));
			ballList[21] = (new Ball(140,80,2,0,1, scale));
			ballList[22] = (new Ball(140,120,2,0,-1, scale));
			ballList[23] = (new Ball(140,160,2,0,1, scale));


			list.addLevel(new Level(24 * 2 * 400 + 24 * 200, -1, ballList, "Hobart",
			999999, 556556, 0, 0,
			30, 300, 10, 10,
			500, 3000, true, 10,
			false, 0, 30000,
			2, 5000, 556556, 556556, 556556, 10000));
	
	
			
					
		
		}
		
		if (name.equals("test1")) {
			
			// bonus lives
			int [] lives = {500, 600, 1000};

			//Number of starting lives
			list = new Scenario(1, lives);
			
			list.title = "testing 1";
			
			list.addLevel(new Level(5000, 5000, null, "test1",
			10000, 556556, 4, 4,
			30, 300, 10, 10,
			500, 3000, true, 3,
			true, 0, 1000,
			2, 5000, 0, 556556, 556556, 10000));
		
		
			list.addLevel(new Level(5000, -1, null, "test1",
			5000, 556556, 1, 4,
			30, 300, 100, -1000,
			500, 3000, true, 25,
			false, 0, 1000,
			2, 0,  5, 556556, 556556, 10000));
		}
		if (name.equals("classic2")) {
			list = new Scenario(5);
			
			list.title = "classic2";
			
			ballList = new Ball [20];

				
						
			//Level 1
			list.addBasicLevel(500, null, "Level 1", 4, 20000, 500);
			
			//Level 2
			ballList = new Ball [6];
			ballList[0] = (new Ball(60,-100,1,0,1, scale));
			ballList[1] = (new Ball(80,-200,1,0,1, scale));
			ballList[2] = (new Ball(100,-300,1,0,1, scale));
			ballList[3] = (new Ball(120,-400,1,0,1, scale));
			ballList[4] = (new Ball(140,-500,1,0,1, scale));
			ballList[5] = (new Ball(160,-600,1,0,1, scale));

			list.addBasicLevel(1500,ballList, "Level 2", 1, 1000, 5000);

			list.addBasicLevel(500,null, "Level 3", 2, 5000, 1000);
			
			ballList = new Ball [3];
			ballList[0] = (new Ball(100,-1000,4,0,1, scale));
			ballList[1] = (new Ball(400,-1000,4,0,-1, scale));
			ballList[2] = (new Ball(600,-1000,4,0,0, scale));
			list.addBasicLevel(10000, ballList, "Level 4", 4, 20000, 20000);
		
			list.addBasicLevel(20000, null, "Level 5", 1, 15000, 20000);
		}
		return list;
	}

}